//------------------------------------------------------------------------------
/*
 * Project name:
     LC Meter
 * Copyright:
     Copyright  2009 Ar2r. All rights reserved.
 * Description:
     PIC counts input, calculates and show result to display.
 * Test configuration:
     MCU:             PIC16F628
     Dev.Board:       Custom
     Oscillator:      HS, 4.194304 MHz
     Ext. Modules:    LCD 1x16
     SW:              MikroC v8.2
 * NOTES:
     None.
*/
//------------------------------------------------------------------------------
#include "Ar2r.h"

unsigned int   cntr;
unsigned long  freq, F2, temp;            // Number of RB0 transition ; frequency
float          cap, ind, vr;
float          Cx, Lx, Kfr1, Kfr2;
unsigned short state;                     // If interrupt occured =1
unsigned char  ch, vrc, *vrs, c2;
//------------------------------------------------------------------------------
void ShowRes(float res, unsigned char *str, unsigned char cm)
{
    unsigned char stri[6], ste[8], c1;
    unsigned short z=0;
    WriteLCD(1, 3, str);
    while(res<1 && z<4)
        {
            res*=1000;
            z++;
        }
    temp=res*100;
    WordToStr(temp, stri);
    ste[0]=stri[0];
    ste[1]=stri[1];
    ste[2]=stri[2];
    if(ste[2]==32) ste[2]='0';
    ste[3]='.';
    ste[4]=stri[3];
    if(ste[4]==32) ste[4]='0';
    ste[5]=stri[4];
    switch(z)
        {
            case 0: c1=' '; break;
            case 1: c1='m'; break;    // m
            case 2: c1=228; break;    // u
            case 3: c1='n'; break;    // n
            case 4: c1='p'; break;    // p
            //default: Lcd_Chr_Cp('E'); break;    // E
        }
    if(cm==' ') c1=' ';
    ste[6]=c1;
    ste[7]=cm;
    WriteLCD(2,1, ste);
}
//------------------------------------------------------------------------------
void Kas(void)
{
    Delay_ms(300);
}
//------------------------------------------------------------------------------
void CalcFreq(void)
{
    freq=(TMR1H<<8)+TMR1L;             // Calculates decimal value of frequency
    freq=(cntr*65536)+freq;
}
//------------------------------------------------------------------------------
void ShowFreq(void)
{
    WriteLCD(1,2, "Freq.= ");          // Print text to LCD for Test mode
    ch=freq/10000000 %10;
    Lcd_Chr(2,1, ch+48);
    ch=freq/1000000 %10;
    Lcd_Chr_Cp(ch+48);
    ch=freq/100000 %10;
    Lcd_Chr_Cp(ch+48);
    ch=freq/10000 %10;
    Lcd_Chr_Cp(ch+48);
    ch=freq/1000 %10;
    Lcd_Chr_Cp(ch+48);
    ch=freq/100 %10;
    Lcd_Chr_Cp(ch+48);
    ch=freq/10 %10;
    Lcd_Chr_Cp(ch+48);
    ch=freq/1 %10;
    Lcd_Chr_Cp(ch+48);
}
//------------------------------------------------------------------------------
void ResetClk(void)
{
    cntr = 0;                          // Clear counters
    TMR1H= 0;
    TMR1L= 0;
    state= 0;

    PORTA.F0 = 1;                      // Reset Clock
    PORTA.F0 = 0;                      // Enable Clock
}
//------------------------------------------------------------------------------
void EnableIRQ(void)
{
    PIE1   = 0b00000001;               // TMR1IE enabled
    INTCON = 0b11010000;               // INTE, PEIE and GIE enabled
}
//------------------------------------------------------------------------------
void DisableIRQ(void)
{
    INTCON.GIE = 0;                    // Stop all interrupts
    PIE1       = 0;
}
//------------------------------------------------------------------------------
void Cal(void)
{
    Kas();
    EnableIRQ();
    ResetClk();                        // Reset external timebase
    while(state==0) {}

    DisableIRQ();
    CalcFreq();                        // Calculates decimal value of frequency
}
//------------------------------------------------------------------------------
void Calibrate(void)
{
    PORTA.F1=1;                        // Activate Reed Relay
    WriteLCD(1,3, "Calibr");           // print string a on LCD
    WriteLCD(2,1, "ating");            // print string a on LCD
    Cal();
    F2=freq;
    
    PORTA.F1=0;                        // Deactivate Reed Relay
    Cal();
    
    Kfr1=Kvad(freq);
    Kfr2=Kvad(F2);
    cap=(Kfr2/(Kfr1-Kfr2))*0.000000001015;
    ind=(1/(39.478417*Kfr1*cap));
}
//------------------------------------------------------------------------------
void interrupt(void)
{
    if(INTCON.INTF)                    // External interrupt occured
        {
            state=1;
            INTCON.INTF = 0;           // Clear interrupt flag to enable next call
        }
    if(PIR1.TMR1IF)                    // TMR1 overfow occured
        {
            cntr++ ;                   // inc. transition counter
            PIR1.TMR1IF = 0 ;          // Clear interrupt flag to enable next call on overflow
        }
}
//------------------------------------------------------------------------------
void main()
{
    Lcd_Config(&PORTB,4,7,0,3,2,1,5);                  // Custom initialize display on PORTB
    TRISB.F0   = 1;                                    // Set PORTB B0 as input (IRQ to stop count and display freq.)
    TRISB.F6   = 1;                                    // Set PORTB B6 as input (count input freq. T1CKI)
    CMCON      = 0b00000111;                           // Set digital input mode for PORTA
    TRISA      = 0b11111100;                           // Set PORTA A0 as output (reset pin)
                                                       // Set PORTA A1 as output (Reed Relay pin)
                                                       // Set PORTA A2 as input (test mode)
                                                       // Set PORTA A3 as input (selection L or C)
                                                       // Set PORTA A4 as input
    OPTION_REG = 0b11000000;
    T1CON      = 0b00000111;

    Lcd_Cmd(LCD_CURSOR_OFF);                           // Turn cursor off
    //Ar2rSplash();                                      // You are GAY if you remove this line :D
    //Lcd_Cmd(LCD_CLEAR);                                // Send command  to LCD (clear LCD)

    Calibrate();

    EnableIRQ();
    ResetClk();                                        // Reset external timebase

    while (1)                                          // Endless loop
        {
            if(state==1)                               // If external interrupt occured RB0
                {
                    DisableIRQ();
                    CalcFreq();                        // Calculates decimal value of frequency
                    
                    //if(PORTA.F2==0)
                        {
                            //ShowFreq();
                        }
                    //else
                        {
                            if(PORTA.F3==1)
                                {
                                    vr=cap;
                                    vrc='F';
                                    vrs="Cap.=";
                                }
                            else
                                {
                                    vr=ind;
                                    vrc='H';
                                    vrs="Ind.=";
                                }
                            Cx=((Kfr1/Kvad(freq))-1)*vr;
                            if(Cx<0)
                                {
                                    c2='E';
                                    Cx=0;
                                    vrc=' ';
                                }
                            else c2=' ';
                            if(freq<4800)
                                {
                                    Cx=0;
                                    c2='!';
                                    vrc=' ';
                                }
                            vrs[5]=c2;
                            ShowRes((Cx), vrs, vrc);
                        }

                    EnableIRQ();
                    ResetClk();                        // Reset external timebase
                }
        }
}
//------------------------------------------------------------------------------
